from closure_linter import errors
from closure_linter import errorrules

OriginalShouldReportError = None

def InjectErrorReporter():
    global OriginalShouldReportError
    OriginalShouldReportError = errorrules.ShouldReportError
    errorrules.ShouldReportError = ShouldReportError

def ShouldReportError(error):
    global OriginalShouldReportError
    return error not in (errors.LINE_TOO_LONG,) and OriginalShouldReportError(error)


from pipes import quote
import os, time, glob, hashlib
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

class BaseBuilder:

    def __init__(self, project_path):
        self.__compiler_args = []
        self.project_path = project_path
        self.__init_closure_path()

    def __init_closure_path(self):
        closure_paths_file = os.path.join(self.project_path, '.closure_paths')
        if not os.path.exists(closure_paths_file):
            raise Exception('Closure paths file not found. Please call bootstrap() before using builders')
        closure_paths = open(closure_paths_file, 'r').read()
        closure_paths = os.path.join(self.project_path, closure_paths)
        if not os.path.exists(closure_paths):
            raise Exception('Closure library not found. Please call bootstrap() before using builders')
        self.closure_base_path = closure_paths

    def set_compiler_path(self, path):
        self.__compiler_path = quote(path)

    def add_compiler_arg(self, name, value = None):
        self.__compiler_args.append(name)
        if value is not None:
            self.__compiler_args.append(value)

    def get_compiler_args_str(self):
        if len(self.__compiler_args) > 0:
            return ' '.join([quote(arg) for arg in self.__compiler_args])
        return ''

    def get_compiler_args(self):
        return self.__compiler_args

    def build(self, fail_on_error=True):
        pass


class ChangeEventHandler(FileSystemEventHandler):

    def __init__(self, builder):
        self.__builder = builder
        FileSystemEventHandler.__init__(self)

    def on_any_event(self, event):
        self.__builder.build_changes()


class BaseObservableBuilder(BaseBuilder):

    __built = False # will be implemented a bit later
    __hashes = {}

    def __has_changes(self):
        if not self.__built:
            return True
        new_hashes = self.__get_hashes()
        if len(set(self.__hashes.keys()) - set(new_hashes.keys())) > 0:
            return True

        for path in new_hashes.keys():
            if self.__hashes[path] != new_hashes[path]:
                return True

        return False

    def __hash(self, path):
        f = open(path, 'rb')
        step = 65536
        buf = f.read(step)
        hasher = hashlib.sha256()
        while len(buf) > 0:
            hasher.update(buf)
            buf = f.read(step)
        return hasher.digest()

    def __get_hashes(self):
        res = {}
        for f in self.get_watch_targets():
            if not f.startswith(self.project_path):
                f = os.path.join(self.project_path, f)
            paths = glob.glob(f)
            for path in paths:
                res[path] = self.__hash(path)
        return res

    def build_changes(self):
        if self.__has_changes():
            self.build(fail_on_error=False)

    def build_complete(self):
        self.__built = True
        self.__hashes = self.__get_hashes()

    def get_watch_targets(self):
        raise Exception('Not implemented')

    def watch(self):
        self.build(fail_on_error=False)
        event_handler = ChangeEventHandler(self)

        folders = []
        for input in self.get_watch_targets():
            folder = os.path.dirname(os.path.join(self.project_path, input))
            if not folder in folders:
                folders.append(folder)

        observer = Observer()
        for input in folders:
            observer.schedule(event_handler, input, recursive=True)
        observer.start()

        return observer

    def watch_forever(self):
        observer = self.watch()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()



__author__ = 'alex'
import os

from fabric.api import local, hide
from ..base.base_builder import BaseBuilder

class Linter(BaseBuilder):

    def __init__(self, project_path, strict = True, ignore_80_symbols = False):
        BaseBuilder.__init__(self, project_path)
        self.__sources = []
        self.__excludes = []
        self.__ignore_80_symbols = ignore_80_symbols

        if strict:
            self.add_compiler_arg('--strict')

    def add_sources(self, sources):
        self.__sources.append(sources)

    def add_exclude(self, exclude):
        self.__excludes.append(exclude)

    def __get_excludes_arg(self):
        if len(self.__excludes) == 0: return ''
        return ' -x %s' % ','.join([os.path.join(self.project_path, ex) for ex in self.__excludes])

    def __get_sources_arg(self):
        if len(self.__sources) == 0:
            raise Exception('No sources specified')

        return ' -r %s' % ','.join([os.path.join(self.project_path, src) for src in self.__sources])

    def __get_args(self):
        args = self.get_compiler_args_str()
        args += self.__get_sources_arg()
        args += self.__get_excludes_arg()
        return args

    def autofix(self):
        BaseBuilder.build(self)

        executable = self.__get_autofix_executable()

        local('%s %s' % (executable, self.__get_args()))

    def lint(self):
        BaseBuilder.build(self)

        executable = self.__get_linter_executable()

        print 'Running linter...'

        with hide('running'):
            local('%s %s' % (executable, self.__get_args()))

    def __get_linter_executable(self):
        if self.__ignore_80_symbols:
            return 'python %s' % os.path.join(os.path.dirname(__file__), '..', 'gjslint_ext', 'linter.py')
        else:
            return 'gjslint'

    def __get_autofix_executable(self):
        if self.__ignore_80_symbols:
            return 'python %s' % os.path.join(os.path.dirname(__file__), '..', 'gjslint_ext', 'autofix.py')
        else:
            return 'fixjsstyle'



__author__ = 'alex'

from parts.stylesheets import StylesheetsBuilder
from parts.deps import DepsBuilder
from parts.linter import Linter
from parts.js import JSBuilder
from bootstrap import bootstrap
from parts.templates import TemplatesBuilder
from server import serve


import bootstrap, os
from SocketServer import TCPServer
from SimpleHTTPServer import SimpleHTTPRequestHandler

class CustomServer(TCPServer):
    def __init__(self, project_path, server_address, RequestHandlerClass, deps_builder=None, js_builder=None,html_folder='.'):
        self.html_folder = html_folder
        self.project_path = project_path
        self.deps_builder = deps_builder
        self.js_builder = js_builder
        TCPServer.__init__(self, server_address, RequestHandlerClass)


class RequestHandler(SimpleHTTPRequestHandler):
    def __get_google_closure_file(self):
        path = os.path.relpath(bootstrap.get_paths(self.server.project_path), self.server.project_path)
        path = os.path.join(path, 'google-closure-library', 'closure')
        self.path = path + self.path
        SimpleHTTPRequestHandler.do_GET(self)

    def __get_deps(self):
        deps_content = self.server.deps_builder.get_deps()

        self.send_response(200)
        self.send_header("Content-type", 'text/javascript')
        self.end_headers()

        self.wfile.write(deps_content)

    def do_GET(self):
        if self.path.startswith('/goog/'):
            self.__get_google_closure_file()
        elif self.path == '/deps' and not (self.server.deps_builder is None):
            self.__get_deps()
        elif self.server.js_builder is not None and self.path == ('/'+self.server.js_builder.get_output_file()):
            self.server.js_builder.build_changes()
            SimpleHTTPRequestHandler.do_GET(self)
        else:
            if os.path.exists(self.server.project_path + '/' + self.server.html_folder + '/' + self.path):
                self.path = self.server.html_folder + '/' + self.path
            SimpleHTTPRequestHandler.do_GET(self)


def serve(project_path, port=8000, html_folder='.', deps_builder=None, stylesheets_builder=None, templates_builder=None,
          js_builder=None):
    httpd = CustomServer(project_path, ("", port), RequestHandler, deps_builder, js_builder, html_folder)

    stylesheets_observer = None
    templates_observer = None

    if stylesheets_builder is not None:
        stylesheets_observer = stylesheets_builder.watch()

    if templates_builder is not None:
        templates_observer = templates_builder.watch()

    print 'Server started: http://127.0.0.1:%s/' % port
    print 'Start monitoring for changes'
    httpd.serve_forever()

    if stylesheets_observer is not None:
        stylesheets_observer.stop()
        stylesheets_observer.join()

    if templates_observer is not None:
        templates_observer.stop()
        templates_observer.join()


__author__ = 'alex'
import os
from fabric.api import local, hide, settings
from ..base.base_builder import BaseObservableBuilder

class StylesheetsBuilder(BaseObservableBuilder):

    def __init__(self, project_path):
        self.__inputs = []
        BaseObservableBuilder.__init__(self, project_path)

    def set_output_file(self, path):
        self.__output_file = path

    def add_stylesheet(self, stylesheet):
        self.__inputs.append(stylesheet)

    def get_watch_targets(self):
        return self.__inputs

    def build(self, fail_on_error=True):
        BaseObservableBuilder.build(self)

        if self.__output_file is None:
            raise Exception('output_file required')
        if len(self.__inputs) == 0:
            raise Exception('No sources')

        print 'Building stylesheets... '

        builder_path = os.path.join(self.closure_base_path, 'google-closure-stylesheets', 'closure-stylesheets.jar')

        args = ''
        args += self.get_compiler_args_str()
        args += ' --output-file %s' % os.path.join(self.project_path, self.__output_file)
        args += ' '+' '.join([os.path.join(self.project_path, src) for src in self.__inputs])

        with hide('running'):
            with settings(warn_only=not fail_on_error):
                local('java -jar %s %s' % (builder_path, args))

        self.build_complete()



import os
from pipes import quote
from fabric.api import local, hide
from ..base.base_builder import BaseBuilder


class DepsBuilder(BaseBuilder):

    def set_output_file(self, path):
        self.__output_file = path

    def set_source(self, source):
        self.__source = source

    def set_custom_path_prefix(self, path):
        self.__custom_path_prefix = path

    def build(self):
        BaseBuilder.build(self)
        if self.__output_file is None:
            raise Exception('output file not specified')

        print 'Building deps... '

        with hide('running'):
            local('python %s %s' % (self.__get_deps_writer(), self.__get_args()))

    def get_deps(self):
        BaseBuilder.build(self)

        with hide('running'):
            res = local('python %s %s' % (self.__get_deps_writer(), self.__get_args(add_output=False)), capture=True)
        return res

    def __get_deps_writer(self):
        return os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'bin', 'build', 'depswriter.py')

    def __get_args(self, add_output = True):
        args = ''
        args += self.get_compiler_args_str()
        if add_output:
            args += ' --output_file=%s' % os.path.join(self.project_path, self.__output_file)
        if not(self.__source is None):
            folder = os.path.join(self.project_path, self.__source)
            args += " --root_with_prefix='%s %s'" % (quote(folder), quote(self.__get_path_prefix()))
        return args

    def __get_path_prefix(self):
        if self.__custom_path_prefix is None:
            base_js_path = os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'goog')
            folder = os.path.join(self.project_path, self.__source)
            return os.path.relpath(folder, base_js_path)
        else:
            return self.__custom_path_prefix



__author__ = 'alex'

import os, urllib, zipfile
import platform
from fabric.api import local
from pipes import quote
from shutil import rmtree

"""
Module for installing closure library, templates, stylesheets and plovr into project.
"""

CLOSURE_LIBRARY_SVN = 'http://closure-library.googlecode.com/svn/trunk/'
CLOSURE_STYLESHEETS = 'http://closure-stylesheets.googlecode.com/files/closure-stylesheets-20111230.jar'
CLOSURE_TEMPLATES = 'http://closure-templates.googlecode.com/files/closure-templates-for-javascript-latest.zip'
CLOSURE_COMPILER = 'http://closure-compiler.googlecode.com/files/compiler-latest.zip'
PLOVR = 'http://plovr.googlecode.com/files/plovr-eba786b34df9.jar'

def __svn_up(path):
    try:
        local('svn up %s' % path)
    except StandardError:
        raise StandardError('svn not found. Please install console svn')

def __svn_checkout(url, path):
    try:
        local('svn checkout %s %s' % (url, quote(path)))
    except StandardError:
        raise StandardError('svn not found. Please install console svn')

def __download_and_unzip(url, path, dir_name):
    zip_path = os.path.join(path, dir_name + '.zip')

    if os.path.exists(zip_path):
        os.unlink(zip_path)

    urllib.urlretrieve(
        url,
        zip_path
    )

    target_path = os.path.join(path, dir_name)
    if os.path.exists(target_path):
        rmtree(target_path)

    zip = zipfile.ZipFile(zip_path)
    zip.extractall(path=target_path)
    zip.close()

    os.remove(zip_path)

def __append_to_gitignore(ignore_string, project_path):
    gitignore = os.path.join(project_path, '.gitignore')
    ignore_string += '\n'
    if os.path.exists(gitignore):
        content = open(gitignore, 'r').read()
        if not ignore_string in content:
            open(gitignore, "a").write(ignore_string)
    else:
        open(gitignore, "w").write(ignore_string)

def update_closure_library(path):
    lib_path = os.path.join(path, 'google-closure-library')
    if platform.system() == 'Windows':
        lib_path = '/'.join(lib_path.split('\\'))
    if os.path.exists(lib_path) and os.path.exists(os.path.join(lib_path,'.svn')):
        __svn_up(lib_path)
    else:
        __svn_checkout(CLOSURE_LIBRARY_SVN, lib_path)

def install_closure_stylesheets(path):
    print('Installing google closure stylesheets from %s' % CLOSURE_STYLESHEETS)
    stylesheets_path = os.path.join(path, 'google-closure-stylesheets')
    if os.path.exists(stylesheets_path):
        rmtree(stylesheets_path)
    os.mkdir(stylesheets_path)
    urllib.urlretrieve(
        CLOSURE_STYLESHEETS,
        os.path.join(stylesheets_path, 'closure-stylesheets.jar')
    )

def install_closure_compiler(path):
    print('Installing google closure compiler from %s' % CLOSURE_COMPILER)
    __download_and_unzip(CLOSURE_COMPILER, path, 'google-closure-compiler')

def install_plovr(path):
    print('Installing plovr from %s' % PLOVR)
    plovr_path = os.path.join(path, 'plovr')
    if os.path.exists(plovr_path):
        rmtree(plovr_path)
    os.mkdir(plovr_path)
    urllib.urlretrieve(
        PLOVR,
        os.path.join(plovr_path, 'plovr.jar')
    )

def install_closure_templates(path):
    print('Installing google closure templates from %s' % CLOSURE_TEMPLATES)
    __download_and_unzip(CLOSURE_TEMPLATES, path, 'google-closure-templates')

def get_paths(project_path):
    paths_file = os.path.join(project_path, '.closure_paths')
    paths = open(paths_file, 'r').read()
    return os.path.join(project_path, paths)

def bootstrap(project_path, dir_name=None, compiler=True, templates=True, stylesheets=True, library=True, plovr=False):
    """
    Setup google closure templates, stylesheets and library into project.
    """

    paths_file = os.path.join(project_path, '.closure_paths')
    if dir_name is None:
        if os.path.exists(paths_file):
            dir_name = open(paths_file, 'r').read()
        else:
            dir_name = 'google_closure'

    path = os.path.join(project_path, dir_name)
    if not os.path.exists(path):
        os.mkdir(path)

    if compiler:
        install_closure_compiler(path)
    if library:
        update_closure_library(path)
    if stylesheets:
        install_closure_stylesheets(path)
    if templates:
        install_closure_templates(path)
    if plovr:
        install_plovr(path)

    with open(paths_file, 'w') as paths:
        paths.write(dir_name)

    if os.path.exists(os.path.join(project_path, '.git')):
        __append_to_gitignore(ignore_string=dir_name + '/', project_path=project_path)
        __append_to_gitignore(ignore_string=paths_file, project_path=project_path)



import sys
import os
from fabric.api import local, hide, settings
from ..base.base_builder import BaseObservableBuilder

class JSBuilder(BaseObservableBuilder):

    def __init__(self, project_path, advanced=True):
        BaseObservableBuilder.__init__(self, project_path)
        if advanced:
            self.add_compiler_arg('--compilation_level', 'ADVANCED_OPTIMIZATIONS')
            self.add_compiler_arg('--define', 'goog.DEBUG=false')

            self.add_compiler_arg('--jscomp_error', 'accessControls')
            self.add_compiler_arg('--jscomp_error', 'ambiguousFunctionDecl')
            # self.add_compiler_arg('--jscomp_error', 'cast')
            self.add_compiler_arg('--jscomp_error', 'checkRegExp')
            self.add_compiler_arg('--jscomp_error', 'checkTypes')
            self.add_compiler_arg('--jscomp_error', 'checkVars')
            self.add_compiler_arg('--jscomp_error', 'const')
            self.add_compiler_arg('--jscomp_error', 'constantProperty')
            self.add_compiler_arg('--jscomp_error', 'deprecated')
            self.add_compiler_arg('--jscomp_error', 'duplicateMessage')
            self.add_compiler_arg('--jscomp_error', 'es5Strict')
            self.add_compiler_arg('--jscomp_error', 'externsValidation')
            self.add_compiler_arg('--jscomp_error', 'fileoverviewTags')
            self.add_compiler_arg('--jscomp_error', 'globalThis')
            self.add_compiler_arg('--jscomp_error', 'internetExplorerChecks')
            self.add_compiler_arg('--jscomp_error', 'invalidCasts')
            self.add_compiler_arg('--jscomp_error', 'misplacedTypeAnnotation')
            self.add_compiler_arg('--jscomp_error', 'missingProperties')
            self.add_compiler_arg('--jscomp_error', 'nonStandardJsDocs')
            self.add_compiler_arg('--jscomp_error', 'suspiciousCode')
            self.add_compiler_arg('--jscomp_error', 'strictModuleDepCheck')
            self.add_compiler_arg('--jscomp_error', 'typeInvalidation')
            self.add_compiler_arg('--jscomp_error', 'undefinedNames')
            self.add_compiler_arg('--jscomp_error', 'undefinedVars')
            self.add_compiler_arg('--jscomp_error', 'unknownDefines')
            self.add_compiler_arg('--jscomp_error', 'uselessCode')
            self.add_compiler_arg('--jscomp_error', 'visibility')

            self.add_compiler_arg('--warning_level', 'VERBOSE')
            self.add_compiler_arg('--summary_detail_level', '3')

            self.add_compiler_arg('--generate_exports')

    def set_output_file(self, path):
        self.__output_file = path

    def get_output_file(self):
        return self.__output_file

    def set_sources_folder(self, path):
        self.__sources_folder = path

    def set_main_file(self, file):
        self.__main_file = file

    def get_watch_targets(self):
        return self.__source_files

    def build(self, fail_on_error=True):
        BaseObservableBuilder.build(self)

        if self.__output_file is None:
            raise Exception('No output file specified')

        if self.__sources_folder is None:
            raise Exception('No sources folder specified')

        if self.__main_file is None:
            raise Exception('No main file specified')

        print 'Building javascript... '

        self.__source_files = []

        sys.path.append(os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'bin'))
        import calcdeps

        js_out = os.path.join(self.project_path, self.__output_file)
        js_src = os.path.join(self.project_path, self.__sources_folder)
        closure_src = os.path.join(self.closure_base_path, 'google-closure-library', 'closure', 'goog')
        closure_compiler = os.path.join(self.closure_base_path, 'google-closure-compiler', 'compiler.jar')

        args = list(self.get_compiler_args())

        args.append('--js_output_file')
        args.append(js_out)

        args.append('--js')
        args.append('%s/deps.js' % closure_src)

        self.__source_files.append('%s/deps.js' % closure_src)

        search_paths = calcdeps.ExpandDirectories([js_src, closure_src])

        sources = calcdeps.CalculateDependencies(search_paths, [os.path.join(js_src, self.__main_file)])
        for src in sources:
            args.append('--js')
            args.append(src)
            self.__source_files.append(src)

        with hide('running'):
            with settings(warn_only=not fail_on_error):
                local('java -jar %s %s' % (closure_compiler, ' '.join(args)))

        self.build_complete()


from setuptools import setup

setup(
    name="google_closure_fabric",
    version="0.0.5",
    author="Aleksandr Batsuev",
    author_email="batsuev@gmail.com",
    description="fabric module for simplifying google closure integration",
    license="BSD",
    keywords="google closure gjslint soy",
    url="https://github.com/batsuev/google-closure-fabric",
    packages=['google_closure_fabric', 'google_closure_fabric.gjslint_ext', 'google_closure_fabric.base', 'google_closure_fabric.parts'],
    long_description=open('README.md', 'r').read(),
    classifiers=[
        "Development Status :: 1 - Planning",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    requires=[
        "fabric",
        "watchdog"
    ],
    dependency_links=[
        "http://closure-linter.googlecode.com/files/closure_linter-latest.tar.gz"
    ],
    scripts=['scripts/gclosureboilerplate']
)



__author__ = 'alex'
import os
import tempfile
from fabric.api import local, hide, settings
from ..base.base_builder import BaseObservableBuilder

class TemplatesBuilder(BaseObservableBuilder):

    def __init__(self, project_path, use_goog=True):
        self.__inputs = []
        self.__deps = []

        BaseObservableBuilder.__init__(self, project_path)
        if use_goog:
            self.add_compiler_arg('--shouldProvideRequireSoyNamespaces')
            self.add_compiler_arg('--shouldGenerateJsdoc')

    def set_output_path_format(self, path):
        self.__output_path_format = path

    def add_template(self, path):
        self.__inputs.append(path)

    def add_dep(self, path):
        self.__deps.append(path)

    def get_watch_targets(self):
        return self.__inputs + self.__deps

    def build(self, fail_on_error=True):
        BaseObservableBuilder.build(self)
        if self.__output_path_format is None:
            raise Exception('output_path_format required')
        if len(self.__inputs) == 0:
            raise Exception('No sources')

        print 'Building templates... '

        with hide('running'):
            with settings(warn_only=not fail_on_error):
                local('java -jar %s %s' % (self.__get_builder_path(), self.__get_args(output_path=os.path.join(self.project_path, self.__output_path_format))))

        self.build_complete()

    def __get_args(self, output_path):
        args = ''
        args += self.get_compiler_args_str()

        args += ' --outputPathFormat %s' % output_path
        args += ' --srcs %s' % ','.join([os.path.join(self.project_path, src) for src in self.__inputs])
        if len(self.__deps) > 0:
            args += ' --deps %s' % ','.join([os.path.join(self.project_path, src) for src in self.__deps])
        return args

    def __get_builder_path(self):
        return os.path.join(self.closure_base_path, 'google-closure-templates', 'SoyToJsSrcCompiler.jar')

    def get_template(self):
        f = tempfile.NamedTemporaryFile(delete=False)
        with hide('running'):
            local('java -jar %s %s' % (self.__get_builder_path(),
                                       self.__get_args(output_path=f.name)))
        res = f.read()
        f.close()
        os.unlink(f.name)
        return res
